@@ -210,7 +210,9 @@ class Site(collections.Mapping, collections.Hashable):
 
     @property
     def to_dict(self):
-        """A dictionary representation for Site that is json serializable."""
+        """
+        Json-serializable dict representation for Site.
+        """
         species_list = []
         for spec, occu in self._species.items():
             if isinstance(spec, Specie):
@@ -219,6 +221,17 @@ class Site(collections.Mapping, collections.Hashable):
                 species_list.append({'element': spec.symbol, 'occu': occu})
         return {'name': self.species_string, 'species': species_list, 'occu': occu, 'xyz':[float(c) for c in self._coords]}
 
+    @staticmethod
+    def from_dict(d):
+        """
+        Create Site from dict representation
+        """
+        atoms_n_occu = {}
+        for sp_occu in d['species']:
+            sp = Specie(sp_occu['element'], sp_occu['oxidation_state']) if 'oxidation_state' in sp_occu else Element(sp_occu['element'])
+            atoms_n_occu[sp] = sp_occu['occu']
+        return Site(atoms_n_occu, d['xyz'])
+
 
 class PeriodicSite(Site):
     """
@@ -226,7 +239,7 @@ class PeriodicSite(Site):
     PeriodicSite includes a lattice system.
     """
 
-    def __init__(self, atoms_n_occu, coords, lattice, to_unit_cell = False, coords_are_cartesian = False):
+    def __init__(self, atoms_n_occu, coords, lattice, to_unit_cell=False, coords_are_cartesian=False):
         """
         Create a periodic site.
         
@@ -244,13 +257,13 @@ class PeriodicSite(Site):
                 Set to True if you are providing cartesian coordinates.  Defaults to False.
         """
         self._lattice = lattice
-        self._fcoords = lattice.get_fractional_coords(coords) if coords_are_cartesian else coords
+        self._fcoords = self._lattice.get_fractional_coords(coords) if coords_are_cartesian else coords
 
         if to_unit_cell:
             for i in range(len(self._fcoords)):
                 self._fcoords[i] = self._fcoords[i] - math.floor(self._fcoords[i])
 
-        c_coords = lattice.get_cartesian_coords(self._fcoords)
+        c_coords = self._lattice.get_cartesian_coords(self._fcoords)
         Site.__init__(self, atoms_n_occu, c_coords)
 
     @property
@@ -296,7 +309,7 @@ class PeriodicSite(Site):
         fcoords = [i - math.floor(i) for i in self._fcoords]
         return PeriodicSite(self._species, fcoords, self._lattice)
 
-    def is_periodic_image(self, other, tolerance = 1e-8):
+    def is_periodic_image(self, other, tolerance=1e-8):
         """
         Returns True if sites are periodic images of each other.
         """
@@ -312,7 +325,7 @@ class PeriodicSite(Site):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def distance_and_image_old(self, other, jimage = None):
+    def distance_and_image_old(self, other, jimage=None):
         """
         .. deprecated:: 1.0
             Use :func:`distance_and_image` instead. This code is kept for information reasons. 
@@ -346,7 +359,7 @@ class PeriodicSite(Site):
             jimage = -np.array(np.around(other._fcoords - self._fcoords), int)
         return np.linalg.norm(self.lattice.get_cartesian_coords(jimage + other._fcoords - self._fcoords)), jimage
 
-    def distance_and_image_from_frac_coords(self, fcoords, jimage = None):
+    def distance_and_image_from_frac_coords(self, fcoords, jimage=None):
         """
         Gets distance between site and a fractional coordinate assuming periodic boundary conditions.
         If the index jimage of two sites atom j is not specified it selects the j image nearest to the i atom
@@ -354,7 +367,7 @@ class PeriodicSite(Site):
         if the index jimage of atom j is specified it returns the distance between
         the i atom and the specified jimage atom, the given jimage is also returned.
         
-        Arguments:
+        Args:
             fcoords:
                 fcoords to get distance from.
             jimage:
@@ -382,7 +395,7 @@ class PeriodicSite(Site):
             return mindist, jimage
         return np.linalg.norm(self.lattice.get_cartesian_coords(jimage + fcoords - self._fcoords)), jimage
 
-    def distance_and_image(self, other, jimage = None):
+    def distance_and_image(self, other, jimage=None):
         """
         Gets distance and instance between two sites assuming periodic boundary conditions.
         If the index jimage of two sites atom j is not specified it selects the j image nearest to the i atom
@@ -390,7 +403,7 @@ class PeriodicSite(Site):
         if the index jimage of atom j is specified it returns the distance between
         the i atom and the specified jimage atom, the given jimage is also returned.
         
-        Arguments:
+        Args:
             other:
                 other site to get distance from.
             jimage:
@@ -404,7 +417,7 @@ class PeriodicSite(Site):
         """
         return self.distance_and_image_from_frac_coords(other._fcoords, jimage)
 
-    def distance(self, other, jimage = None):
+    def distance(self, other, jimage=None):
         """
         Get distance between two sites assuming periodic boundary conditions.
         
@@ -434,30 +447,51 @@ class PeriodicSite(Site):
 
     @property
     def to_dict(self):
+        """
+        Json-serializable dict representation of PeriodicSite.
+        """
         species_list = []
         for spec, occu in self._species.items():
             if isinstance(spec, Specie):
                 species_list.append({'element': spec.symbol, 'occu': occu, 'oxidation_state': spec.oxi_state})
             elif isinstance(spec, Element):
                 species_list.append({'element': spec.symbol, 'occu': occu})
-        return {'label': self.species_string, 'species': species_list, 'occu': occu, 'xyz':[float(c) for c in self._coords], 'abc':[float(c) for c in self._fcoords]}
+        return {'label': self.species_string, 'species': species_list,
+                'occu': occu, 'xyz':[float(c) for c in self._coords],
+                'abc':[float(c) for c in self._fcoords],
+                'lattice': self._lattice.to_dict}
+
+    @staticmethod
+    def from_dict(d):
+        """
+        Create PeriodicSite from dict representation
+        """
+        atoms_n_occu = {}
+        for sp_occu in d['species']:
+            sp = Specie(sp_occu['element'], sp_occu['oxidation_state']) if 'oxidation_state' in sp_occu else Element(sp_occu['element'])
+            atoms_n_occu[sp] = sp_occu['occu']
+        return PeriodicSite(atoms_n_occu, d['abc'], Lattice.from_dict(d['lattice']))
 
 
 class Structure(collections.Sequence, collections.Hashable):
     """
-    Basic Structure object with periodicity. Essentially a sequence of sites having a common lattice.
-    Structure is made to be immutable so that they can function as keys in a dictionary.  
-    Modifications should be done by making a new Structure using the structure_modifier module or your own methods.
-    Structure extends Sequence and Hashable, which means that in many cases, it can be used like any Python sequence.
+    Basic Structure object with periodicity. Essentially a sequence of sites 
+    having a common lattice. Structure is made to be immutable so that they 
+    can function as keys in a dict. Modifications should be done by making a 
+    new Structure using the structure_modifier module or your own methods.
+    Structure extends Sequence and Hashable, which means that in many cases, 
+    it can be used like any Python sequence. Iterating through a structure is
+    equivalent to going through the sites in sequence.
     """
 
     DISTANCE_TOLERANCE = 0.01
 
-    def __init__(self, lattice, atomicspecies, coords, validate_proximity = False, to_unit_cell = False, coords_are_cartesian = False):
+    def __init__(self, lattice, atomicspecies, coords, validate_proximity=False,
+                 to_unit_cell=False, coords_are_cartesian=False):
         """
         Create a periodic structure.
         
-        Arguments:
+        Argus:
             lattice:
                 pymatgen.core.lattice Lattice object signify the lattice.
             atomicspecies:
@@ -465,9 +499,11 @@ class Structure(collections.Sequence, collections.Hashable):
             fractional_coords:
                 list of fractional coordinates of each species.
             validate_proximity:
-                Whether to check if there are sites that are less than 1 Ang apart. Defaults to false.
+                Whether to check if there are sites that are less than 1 Ang 
+                apart. Defaults to False.
             coords_are_cartesian:
-                Set to True if you are providing coordinates in cartesian coordinates. Defaults to false.
+                Set to True if you are providing coordinates in cartesian 
+                coordinates. Defaults to False.
         """
         if len(atomicspecies) != len(coords):
             raise StructureError("The list of atomic species must be of the same length as the list of fractional coordinates.")
@@ -579,26 +615,41 @@ class Structure(collections.Sequence, collections.Hashable):
         constant = 1.660468
         return self.composition.weight / self.volume * constant
 
-    def get_distance(self, i, j, jimage = None):
+    def get_distance(self, i, j, jimage=None):
         """
-        get distance between site i and j assuming periodic boundary conditions
-        if the index jimage of two sites atom j is not specified it selects the j image nearest to the i atom
-        and returns the distance and jimage indices in terms of lattice vector translations
-        if the index jimage of atom j is specified it returns the distance between
-        the i atom and the specified jimage atom, the given jimage is also returned.
+        Get distance between site i and j assuming periodic boundary conditions
+        if the index jimage of two sites atom j is not specified it selects the 
+        j image nearest to the i atom and returns the distance and jimage 
+        indices in terms of lattice vector translations if the index jimage of 
+        atom j is specified it returns the distance between the i atom and the 
+        specified jimage atom, the given jimage is also returned.
+        
+        Args:
+            i:
+                Index of first site
+            j:
+                Index of second site
+            jimage:
+                Number of lattice translations in each lattice direction. Default
+                is None for nearest image.
+        
+        Returns:
+            (distance, jimage)
         """
         return self[i].distance(self[j], jimage)
 
     def get_sites_in_sphere(self, pt, r):
         '''
-        Find all sites within a sphere from the structure. This includes sites in other periodic images.
+        Find all sites within a sphere from the structure. This includes sites 
+        in other periodic images.
         
         Algorithm: 
         
-        1. place sphere of radius r in crystal and determine minimum supercell (parallelpiped) which would
-           contain a sphere of radius r. for this we need the projection of a_1 on a unit vector perpendicular 
-           to a_2 & a_3 (i.e. the unit vector in the direction b_1) to determine how many a_1's it will
-           take to contain the sphere. 
+        1. place sphere of radius r in crystal and determine minimum supercell 
+           (parallelpiped) which would contain a sphere of radius r. for this 
+           we need the projection of a_1 on a unit vector perpendicular 
+           to a_2 & a_3 (i.e. the unit vector in the direction b_1) to determine 
+           how many a_1's it will take to contain the sphere. 
            
            Nxmax = r * length_of_b_1 / (2 Pi)
         
@@ -632,7 +683,7 @@ class Structure(collections.Sequence, collections.Hashable):
             fcoords = site_fcoords + submat
             coords = self._lattice.get_cartesian_coords(fcoords)
             dists = (coords - pts) ** 2
-            dists = np.sqrt(dists.sum(axis = 1))
+            dists = np.sqrt(dists.sum(axis=1))
             withindists = (dists <= r)
             for i in range(n):
                 if withindists[i]:
@@ -642,9 +693,10 @@ class Structure(collections.Sequence, collections.Hashable):
 
     def get_neighbors(self, site, r):
         """
-        Get all neighbors to a site within a sphere of radius r.  Excludes the site itself.
+        Get all neighbors to a site within a sphere of radius r.  Excludes the 
+        site itself.
         
-        Arguments:
+        Args:
             site:
                 site, which is the center of the sphere.
             r:
@@ -655,7 +707,7 @@ class Structure(collections.Sequence, collections.Hashable):
         """
         return [(s, dist) for (s, dist) in self.get_sites_in_sphere(site.coords, r) if site != s]
 
-    def get_all_neighbors(self, r, include_index = False):
+    def get_all_neighbors(self, r, include_index=False):
         """
         Get neighbors for each atom in the unit cell, out to a distance r
         Returns a list of list of neighbors for each site in structure.
@@ -720,7 +772,7 @@ class Structure(collections.Sequence, collections.Hashable):
                 coords = self.lattice.get_cartesian_coords(fcoords)
                 submat = [coords] * n
                 dists = (site_coords - submat) ** 2
-                dists = np.sqrt(dists.sum(axis = 1))
+                dists = np.sqrt(dists.sum(axis=1))
                 withindists = (dists <= r) * (dists > 1e-8)
                 for i in range(n):
                     if include_index & withindists[i]:
@@ -791,7 +843,7 @@ class Structure(collections.Sequence, collections.Hashable):
         sortedsites = sorted(self.sites)
         return Structure(self._lattice, [site.species_and_occu for site in sortedsites], [site.frac_coords for site in sortedsites])
 
-    def interpolate(self, end_structure, nimages = 10):
+    def interpolate(self, end_structure, nimages=10):
         '''
         Interpolate between this structure and end_structure. Useful for construction
         NEB inputs.
@@ -844,19 +896,21 @@ class Structure(collections.Sequence, collections.Hashable):
         return "\n".join(outs)
 
     def __str__(self):
-        outs = ["Structure Summary ({s})".format(s = str(self.composition))]
+        outs = ["Structure Summary ({s})".format(s=str(self.composition))]
         outs.append("Reduced Formula: " + str(self.composition.reduced_formula))
         to_s = lambda x : "%0.6f" % x
         outs.append('abc   : ' + " ".join([to_s(i).rjust(10) for i in self.lattice.abc]))
         outs.append('angles: ' + " ".join([to_s(i).rjust(10) for i in self.lattice.angles]))
-        outs.append("Sites ({i})".format(i = len(self)))
+        outs.append("Sites ({i})".format(i=len(self)))
         for i, site in enumerate(self):
             outs.append(" ".join([str(i + 1), site.species_string, " ".join([to_s(j).rjust(12) for j in site.frac_coords])]))
         return "\n".join(outs)
 
     @property
     def to_dict(self):
-        """Json-friendly, dict representation of Structure"""
+        """
+        Json-serializable dict representation of Structure
+        """
         d = {}
         d['lattice'] = self._lattice.to_dict
         d['sites'] = [site.to_dict for site in self]
@@ -864,9 +918,11 @@ class Structure(collections.Sequence, collections.Hashable):
 
     @staticmethod
     def from_dict(structure_dict):
-        """Reconstitute a Structure object from a dict representation of Structure created using to_dict.
+        """
+        Reconstitute a Structure object from a dict representation of Structure 
+        created using to_dict.
         
-        Arguments:
+        Args:
             structure_dict: 
                 dict representation of structure.
         
@@ -902,13 +958,12 @@ class Composition (collections.Mapping, collections.Hashable):
     Represents a Composition, which is essentially a {element:amount} dictionary.
     
     Works almost completely like a standard python dictionary, except that __getitem__
-    is overridden to return 0 when an element is not found.
+    is overridden to return 0 when an element is not found. (somewhat like a 
+    defaultdict, except it is immutable).
     
     Also adds more convenience methods relevant to compositions, e.g., get_fraction.
     
-    >>> comp = Composition.from_formula("LiFePO4")
-    >>> comp.get_atomic_fraction("Li")
-    0.0
+    >>> comp = Composition("LiFePO4")
     >>> comp.get_atomic_fraction(Element("Li"))
     0.14285714285714285
     >>> comp.num_atoms
@@ -935,18 +990,32 @@ class Composition (collections.Mapping, collections.Hashable):
     """
     special_formulas = {'LiO':'Li2O2', 'NaO':'Na2O2', 'KO':'K2O2', 'HO':'H2O2', 'O':'O2', 'F':'F2', 'N':'N2', 'Cl':'Cl2', 'H':'H2'}
 
-    def __init__(self, elmap):
+    def __init__(self, *args, **kwargs):
         """
+        Very flexible Composition construction, similar to the built-in Python
+        dict(). Also extended to allow simple string init.
+        
         Args:
-            elmap: 
-                a dict of {Element/Specie: float} representing amounts of each element or specie.
+            Any form supported by the Python built-in dict() function.
+            
+            1. A dict of either {Element/Specie: amount}, {string symbol:amount}, 
+               or {atomic number:amount} or any mixture of these. E.g., 
+               {Element("Li"):2 ,Element("O"):1}, {"Li":2, "O":1}, {3:2, 8:1}
+               all result in a Li2O composition.
+            2. Keyword arg initialization, similar to a dict, e.g., 
+               Compostion(Li = 2, O = 1)
+               
+            In addition, the Composition constructor also allows a single string
+            as an input formula. E.g., Composition("Li2O")
+        
         """
+        if len(args) == 1 and isinstance(args[0], basestring):
+            elmap = self._parse_formula(args[0])
+        else:
+            elmap = dict(*args, **kwargs)
         if any([e < 0 for e in elmap.values()]):
             raise ValueError("Amounts in Composition cannot be negative!")
-        for e in elmap.keys():
-            if not isinstance(e, (Element, Specie)):
-                raise TypeError("Keys must be instances of Element or Specie!")
-        self._elmap = elmap.copy()
+        self._elmap = {smart_element_or_specie(k): v for k, v in elmap.items()}
         self._natoms = sum(self._elmap.values())
 
     def __getitem__(self, el):
@@ -969,8 +1038,8 @@ class Composition (collections.Mapping, collections.Hashable):
 
     def __add__(self, other):
         """
-        Adds two compositions.
-        For example, an Fe2O3 composition + an FeO composition gives a Fe3O4 composition.
+        Adds two compositions. For example, an Fe2O3 composition + an FeO 
+        composition gives a Fe3O4 composition.
         """
         new_el_map = {el:self[el] for el in self}
         for k in other.keys():
@@ -983,10 +1052,12 @@ class Composition (collections.Mapping, collections.Hashable):
 
     def __sub__(self, other):
         """
-        Subtracts two compositions.
-        For example, an Fe2O3 composition - an FeO composition gives an FeO2 composition.
-        raises a ValueError if the subtracted composition is greater than the original composition
-        in any of its elements.
+        Subtracts two compositions. For example, an Fe2O3 composition - an FeO 
+        composition gives an FeO2 composition.
+        
+        Raises:
+            ValueError if the subtracted composition is greater than the 
+            original composition in any of its elements.
         """
         new_el_map = {el:self[el] for el in self}
         for k in other.keys():
@@ -1008,7 +1079,8 @@ class Composition (collections.Mapping, collections.Hashable):
 
     def __hash__(self):
         '''
-        Minimally effective hash function that just distinguishes between Compositions with different elements.
+        Minimally effective hash function that just distinguishes between 
+        Compositions with different elements.
         '''
         hashcode = 0
         for el in self._elmap.keys():
@@ -1029,9 +1101,10 @@ class Composition (collections.Mapping, collections.Hashable):
     @property
     def is_element(self):
         '''
-        True if composition is for an element
+        True if composition is for an element.
         '''
-        return len(self._elmap) == 1
+        positive_amts = [amt for amt in self._elmap.values() if amt > self.amount_tolerance]
+        return len(positive_amts) == 1
 
     def copy(self):
         return Composition(self._elmap)
@@ -1039,10 +1112,11 @@ class Composition (collections.Mapping, collections.Hashable):
     @property
     def formula(self):
         '''
-        Returns a formula string, with elements sorted by electronegativity e.g. Li4 Fe4 P4 O16.
+        Returns a formula string, with elements sorted by electronegativity,
+        e.g., Li4 Fe4 P4 O16.
         '''
         elements = self._elmap.keys()
-        elements = sorted(elements, key = lambda el: el.X)
+        elements = sorted(elements, key=lambda el: el.X)
         formula = []
         for el in elements:
             if self[el] != 0:
@@ -1052,10 +1126,11 @@ class Composition (collections.Mapping, collections.Hashable):
     @property
     def alphabetical_formula(self):
         '''
-        Returns a formula string, with elements sorted by alphabetically e.g. Fe4 Li4 O16 P4.
+        Returns a formula string, with elements sorted by alphabetically 
+        e.g. Fe4 Li4 O16 P4.
         '''
         elements = self._elmap.keys()
-        elements = sorted(elements, key = lambda el: el.symbol)
+        elements = sorted(elements, key=lambda el: el.symbol)
         formula = []
         for el in elements:
             if self[el] != 0:
@@ -1075,11 +1150,12 @@ class Composition (collections.Mapping, collections.Hashable):
 
     def get_reduced_formula_and_factor(self):
         '''
-        Returns a pretty normalized formula and a multiplicative factor, i.e., Li4Fe4P4O16 returns (LiFePO4, 4).
+        Returns a pretty normalized formula and a multiplicative factor, i.e., 
+        Li4Fe4P4O16 returns (LiFePO4, 4).
         '''
 
         elements = self._elmap.keys()
-        elements = sorted(elements, key = lambda el: el.X)
+        elements = sorted(elements, key=lambda el: el.X)
         elements = filter(lambda el: self[el] != 0, elements)
         num_el = len(elements)
         contains_polyanion = False
@@ -1166,6 +1242,7 @@ class Composition (collections.Mapping, collections.Hashable):
         Arguments:
             el:
                 Element
+        
         Returns:
             Atomic fraction for element el in Composition
         '''
@@ -1176,17 +1253,18 @@ class Composition (collections.Mapping, collections.Hashable):
         Arguments:
             el:
                 Element
+        
         Returns:
             Weight fraction for element el in Composition
         '''
         return el.atomic_mass * self[el] / self.weight
 
-    @staticmethod
-    def from_formula(formula):
+    def _parse_formula(self, formula):
         '''
         Arguments:
             formula:
                 A string formula, e.g. Fe2O3, Li3Fe2(PO4)3
+        
         Returns:
             Composition with that formula.
         '''
@@ -1212,13 +1290,29 @@ class Composition (collections.Mapping, collections.Hashable):
                 factor = float(m.group(2))
             unit_sym_dict = get_sym_dict(m.group(1), factor)
             expanded_formula = formula.replace(m.group(), "".join([el + str(amt) for el, amt in unit_sym_dict.items()]))
-            return Composition.from_formula(expanded_formula)
-        return Composition.from_dict(get_sym_dict(formula, 1))
+            return self._parse_formula(expanded_formula)
+        return get_sym_dict(formula, 1)
+
+
+    @staticmethod
+    def from_formula(formula):
+        '''
+        .. deprecated:: 1.6.1
+        
+        Use Composition(formula) instead.
+        '''
+        return Composition(formula)
 
     @property
     def anonymized_formula(self):
+        """
+        An anonymized formula. Unique species are arranged in ordering of 
+        increasing amounts and assigned ascending alphabets. Useful for 
+        prototyping formulas. For example, all stoichiometric perovskites have
+        anonymized_formula ABC3.
+        """
         reduced_comp = self.get_reduced_composition_and_factor()[0]
-        els = sorted(reduced_comp.elements, key = lambda e: reduced_comp[e])
+        els = sorted(reduced_comp.elements, key=lambda e: reduced_comp[e])
         ascii_code = 65
         anon_formula = []
         for e in els:
@@ -1238,15 +1332,19 @@ class Composition (collections.Mapping, collections.Hashable):
         return "Comp: " + self.formula
 
     @staticmethod
-    def from_dict(sym_dict):
+    def from_dict(d):
         '''
-        Arguments:
-            sym_dict:
-                A element symbol: amount dict, e.g. {"Fe":2, "O":3}
-        Returns:
-            Composition with that formula.
+        Creates a composition from a dict generated by to_dict. Strictly not 
+        necessary given that the standard constructor already takes in such an
+        input, but this method preserves the standard pymatgen API of having
+        from_dict methods to reconstitute objects generated by to_dict. Allows
+        for easier introspection.
+        
+        Args:
+            d:
+                {symbol: amount} dict.
         '''
-        return Composition({Element(sym) : amt for sym, amt in sym_dict.items()})
+        return Composition(d)
 
     @property
     def to_dict(self):
@@ -1267,17 +1365,20 @@ class Composition (collections.Mapping, collections.Hashable):
         return c.to_dict
 
     @staticmethod
-    def ranked_compositions_from_indeterminate_formula(fuzzy_formula, lock_if_strict = True):
+    def ranked_compositions_from_indeterminate_formula(fuzzy_formula, lock_if_strict=True):
         '''
-        Takes in a formula where capitilization might not be correctly entered, and suggests a ranked list of potential Composition matches.
+        Takes in a formula where capitilization might not be correctly entered, 
+        and suggests a ranked list of potential Composition matches.
         Author: Anubhav Jain
         
         Args:
             fuzzy_formula:
-                A formula string, such as 'co2o3' or 'MN', that may or may not have multiple interpretations
+                A formula string, such as 'co2o3' or 'MN', that may or may not 
+                have multiple interpretations
             lock_if_strict:
-                If true, a properly entered formula will only return the one correct interpretation. For example,
-                'Co1' will only return 'Co1' if true, but will return both 'Co1' and 'C1 O1' if false.
+                If true, a properly entered formula will only return the one 
+                correct interpretation. For example, 'Co1' will only return 
+                'Co1' if true, but will return both 'Co1' and 'C1 O1' if false.
         
         Returns:
             A ranked list of potential Composition matches
@@ -1296,27 +1397,32 @@ class Composition (collections.Mapping, collections.Hashable):
         #remove duplicates
         all_matches = list(set(all_matches))
         #sort matches by rank descending
-        all_matches = sorted(all_matches, key = lambda match:match[1], reverse = True)
+        all_matches = sorted(all_matches, key=lambda match:match[1], reverse=True)
         all_matches = [m[0] for m in all_matches]
         return all_matches
 
     @staticmethod
-    def _recursive_compositions_from_fuzzy_formula(fuzzy_formula, m_dict = {}, m_points = 0, factor = 1):
+    def _recursive_compositions_from_fuzzy_formula(fuzzy_formula, m_dict={}, m_points=0, factor=1):
         '''
-        A recursive helper method for formula parsing that helps in interpreting and ranking indeterminate formulas
+        A recursive helper method for formula parsing that helps in interpreting 
+        and ranking indeterminate formulas.
         Author: Anubhav Jain
         
         Arguments:
             fuzzy_formula:
-                A formula string, such as 'co2o3' or 'MN', that may or may not have multiple interpretations
+                A formula string, such as 'co2o3' or 'MN', that may or may not 
+                have multiple interpretations.
             m_dict:
                 A symbol:amt dictionary from the previously parsed formula
             m_points:
                 Number of points gained from the previously parsed formula
             factor:
-                Coefficient for this parse, e.g. (PO4)2 will feed in PO4 as the fuzzy_formula with a coefficient of 2
+                Coefficient for this parse, e.g. (PO4)2 will feed in PO4 as the 
+                fuzzy_formula with a coefficient of 2
+        
         Returns:
-            A list of tuples, with the first element being a Composition and the second element being the number of points awarded that Composition intepretation
+            A list of tuples, with the first element being a Composition and 
+            the second element being the number of points awarded that Composition intepretation
         '''
 
         def _parse_chomp_and_rank(m, f, m_dict, m_points):
@@ -1384,10 +1490,10 @@ class Composition (collections.Mapping, collections.Hashable):
                 mp_dict = dict(m_dict)
                 mp_factor = 1 if mp.group(2) == "" else float(mp.group(2))
                 #match the stuff inside the parenthesis with the appropriate factor
-                for match in Composition._recursive_compositions_from_fuzzy_formula(mp.group(1), mp_dict, mp_points, factor = mp_factor):
+                for match in Composition._recursive_compositions_from_fuzzy_formula(mp.group(1), mp_dict, mp_points, factor=mp_factor):
                     only_me = True
                     #match the stuff outside the parentheses and return the sum
-                    for match2 in Composition._recursive_compositions_from_fuzzy_formula(mp_form.replace(mp.group(), " ", 1), mp_dict, mp_points, factor = 1):
+                    for match2 in Composition._recursive_compositions_from_fuzzy_formula(mp_form.replace(mp.group(), " ", 1), mp_dict, mp_points, factor=1):
                         only_me = False
                         yield (match[0] + match2[0], match[1] + match2[1])
                     #if the stuff inside the parenthesis is nothing, then just return the stuff inside the parentheses
