@@ -16,6 +16,7 @@ __status__ = "Production"
 __date__ = "Sep 23, 2011"
 
 import os
+import glob
 import re
 import math
 import itertools
@@ -28,14 +29,15 @@ import ConfigParser
 import numpy as np
 from numpy.linalg import det
 
-import pymatgen.command_line.aconvasp_caller as aconvasp_caller
 from pymatgen.core.design_patterns import Enum
 from pymatgen.io.io_abc import VaspInput
 from pymatgen.util.string_utils import str_aligned, str_delimited
 from pymatgen.util.io_utils import file_open_zip_aware, clean_lines, micro_pyawk, clean_json
 from pymatgen.core.structure import Structure, Composition
 from pymatgen.core.periodic_table import Element
-from pymatgen.core.electronic_structure import CompleteDos, Dos, PDos, Spin, Orbital, Bandstructure, get_reconstructed_band_structure
+from pymatgen.electronic_structure.core import Spin, Orbital
+from pymatgen.electronic_structure.dos import CompleteDos, Dos, PDos
+from pymatgen.electronic_structure.band_structure.band_structure import BandStructureSymmLine, get_reconstructed_band_structure
 from pymatgen.core.lattice import Lattice
 import pymatgen
 
@@ -48,7 +50,7 @@ class Poscar(VaspInput):
     dynamcics POSCAR files.
     """
 
-    def __init__(self, struct, comment = None, selective_dynamics = None, true_names = True):
+    def __init__(self, struct, comment=None, selective_dynamics=None, true_names=True):
         """        
         Arguments:
             struct:
@@ -131,7 +133,7 @@ class Poscar(VaspInput):
             return Poscar.from_string(f.read(), names)
 
     @staticmethod
-    def from_string(data, default_names = None):
+    def from_string(data, default_names=None):
         """
         Reads a Poscar from a string.
         The code will try its best to determine the elements in the POSCAR in the following order:
@@ -226,7 +228,7 @@ class Poscar(VaspInput):
     def true_names(self):
         return self._true_names
 
-    def get_string(self, direct = True, vasp4_compatible = False):
+    def get_string(self, direct=True, vasp4_compatible=False):
         """
         Returns a string to be written as a POSCAR file. By default, site symbols are written
         which means compatibilty is for vasp >= 5.
@@ -306,7 +308,7 @@ class Incar(dict, VaspInput):
     essentially consists of a dictionary with some helper functions
     """
 
-    def __init__(self, params = dict()):
+    def __init__(self, params=dict()):
         """
         Creates an Incar object.
         
@@ -336,7 +338,7 @@ class Incar(dict, VaspInput):
             i[k] = v
         return i
 
-    def get_string(self, sort_keys = False, pretty = False):
+    def get_string(self, sort_keys=False, pretty=False):
         """
         Returns a string representation of the INCAR.  The reason why this method is
         different from the __str__ method is to provide options for pretty printing.
@@ -368,7 +370,7 @@ class Incar(dict, VaspInput):
             return str_delimited(lines, None, " = ")
 
     def __str__(self):
-        return self.get_string(sort_keys = True, pretty = False)
+        return self.get_string(sort_keys=True, pretty=False)
 
     def write_file(self, filename):
         """
@@ -505,10 +507,10 @@ class Kpoints(VaspInput):
     """
     supported_modes = Enum(("Gamma", "Monkhorst", "Automatic", "Line_mode", "Cartesian", "Reciprocal"))
 
-    def __init__(self, comment = "Default gamma", num_kpts = 0, style = supported_modes.Gamma,
-                 kpts = [[1, 1, 1]], kpts_shift = (0, 0, 0),
-                 kpts_weights = None, coord_type = None, labels = None,
-                 tet_number = 0, tet_weight = 0, tet_connections = None):
+    def __init__(self, comment="Default gamma", num_kpts=0, style=supported_modes.Gamma,
+                 kpts=[[1, 1, 1]], kpts_shift=(0, 0, 0),
+                 kpts_weights=None, coord_type=None, labels=None,
+                 tet_number=0, tet_weight=0, tet_connections=None):
         """
         Highly flexible constructor for Kpoints object.  The flexibility comes at the cost of usability and in 
         general, it is recommended that you use the default constructor only if you know exactly what you are doing
@@ -576,10 +578,10 @@ class Kpoints(VaspInput):
         Returns:
             Kpoints object
         """
-        return Kpoints("Fully automatic kpoint scheme", 0, style = Kpoints.supported_modes.Automatic, kpts = [[subdivisions]])
+        return Kpoints("Fully automatic kpoint scheme", 0, style=Kpoints.supported_modes.Automatic, kpts=[[subdivisions]])
 
     @staticmethod
-    def gamma_automatic(kpts = (1, 1, 1), shift = (0, 0, 0)):
+    def gamma_automatic(kpts=(1, 1, 1), shift=(0, 0, 0)):
         """
         Convenient static constructor for an automatic Gamma centered Kpoint grid.
         
@@ -592,10 +594,10 @@ class Kpoints(VaspInput):
         Returns:
             Kpoints object
         """
-        return Kpoints("Automatic kpoint scheme", 0, Kpoints.supported_modes.Gamma, kpts = [kpts], kpts_shift = shift)
+        return Kpoints("Automatic kpoint scheme", 0, Kpoints.supported_modes.Gamma, kpts=[kpts], kpts_shift=shift)
 
     @staticmethod
-    def monkhorst_automatic(kpts = (2, 2, 2), shift = (0, 0, 0)):
+    def monkhorst_automatic(kpts=(2, 2, 2), shift=(0, 0, 0)):
         """
         Convenient static constructor for an automatic Monkhorst pack Kpoint grid.
         
@@ -608,25 +610,45 @@ class Kpoints(VaspInput):
         Returns:
             Kpoints object
         """
-        return Kpoints("Automatic kpoint scheme", 0, Kpoints.supported_modes.Monkhorst, kpts = [kpts], kpts_shift = shift)
+        return Kpoints("Automatic kpoint scheme", 0, Kpoints.supported_modes.Monkhorst, kpts=[kpts], kpts_shift=shift)
 
     @staticmethod
-    def monkhorst_automatic_density(structure, kppa):
-        """
-            The approach we took here is to use aconvasp to get the kpoint divisions
-        """
-        div = aconvasp_caller.get_num_division_kpoints(structure, kppa)
+    def automatic_density(structure, kppa):
+        '''
+        Writes out a KPOINTS file using the fully automated grid method. Uses Gamma centered meshes 
+        for hexagonal cells and Monkhorst-Pack grids otherwise.
+        
+        Algorithm: 
+            Uses a simple approach scaling the number of divisions along each 
+            reciprocal lattice vector proportional to its length. 
+        '''
 
-        k = None
+        latt = structure.lattice
+        lengths = latt.abc
+        ngrid = kppa / structure.num_sites
 
-        #FIXME This makes little sense to ask for monkhorst_automatic and it' returns gamma centered mesh
+        mult = (ngrid * lengths[0] * lengths[1] * lengths[2]) ** (1 / 3)
 
-        #check if hexagonal!!!!
-        if(Kpoints._is_hexagonal(structure) == True):
-            k = Kpoints.gamma_automatic(div, shift = (0, 0, 0))
-        else:
-            k = Kpoints.monkhorst_automatic(div)
-        return k
+        num_div = [int(round(1 / lengths[i] * mult)) for i in xrange(3)]
+        #ensure that numDiv[i] > 0
+
+        num_div = [i if i > 0 else 1 for i in num_div]
+
+        angles = latt.angles
+        hex_angle_tol = 5 #in degrees
+        hex_length_tol = 0.01 #in angstroms
+        right_angles = [i for i in xrange(3) if abs(angles[i] - 90) < hex_angle_tol]
+        hex_angles = [i for i in xrange(3) if abs(angles[i] - 60) < hex_angle_tol or abs(angles[i] - 120) < hex_angle_tol]
+
+        is_hexagonal = (len(right_angles) == 2 and len(hex_angles) == 1 and abs(lengths[right_angles[0]] == lengths[right_angles[1]]) < hex_length_tol)
+
+        style = Kpoints.supported_modes.Gamma
+        if not is_hexagonal:
+            num_div = [i + i % 2 for i in num_div]
+            style = Kpoints.supported_modes.Monkhorst
+        comment = "pymatgen generated Materials Project kpoints with grid density = " + str(kppa) + ' per atom.'
+        num_kpts = 0
+        return Kpoints(comment, num_kpts, style, [num_div], [0, 0, 0])
 
     @staticmethod
     def _is_hexagonal(structure):
@@ -694,7 +716,7 @@ class Kpoints(VaspInput):
             style = Kpoints.supported_modes.Cartesian if style in "ck" else Kpoints.supported_modes.Reciprocal
             kpts = [[float(x) for x in lines[i].split()] for i in xrange(3, 6)]
             kpts_shift = [float(x) for x in lines[6].split()]
-            return Kpoints(comment = comment, num_kpts = num_kpts, style = style, kpts = kpts, kpts_shift = kpts_shift)
+            return Kpoints(comment=comment, num_kpts=num_kpts, style=style, kpts=kpts, kpts_shift=kpts_shift)
 
         #Line-mode KPOINTS, usually used with band structures
         if style == 'l':
@@ -709,8 +731,8 @@ class Kpoints(VaspInput):
                 if m:
                     kpts.append([float(m.group(1)), float(m.group(2)), float(m.group(3))])
                     labels.append(m.group(4).strip())
-            return Kpoints(comment = comment, num_kpts = num_kpts, style = style,
-                 kpts = kpts, coord_type = coord_type, labels = labels)
+            return Kpoints(comment=comment, num_kpts=num_kpts, style=style,
+                 kpts=kpts, coord_type=coord_type, labels=labels)
 
         #Assume explicit KPOINTS if all else fails.
         style = Kpoints.supported_modes.Cartesian if style == "ck" else Kpoints.supported_modes.Reciprocal
@@ -737,9 +759,9 @@ class Kpoints(VaspInput):
         except:
             pass
 
-        return Kpoints(comment = comment, num_kpts = num_kpts, style = style,
-                 kpts = kpts, kpts_weights = kpts_weights,
-                 tet_number = tet_number, tet_weight = tet_weight, tet_connections = tet_connections)
+        return Kpoints(comment=comment, num_kpts=num_kpts, style=style,
+                 kpts=kpts, kpts_weights=kpts_weights,
+                 tet_number=tet_number, tet_weight=tet_weight, tet_connections=tet_connections)
 
     def write_file(self, filename):
         """
@@ -799,7 +821,7 @@ class Kpoints(VaspInput):
         kpts_shift = d.get('usershift', [0, 0, 0])
         num_kpts = d.get('nkpoints', 0)
         #coord_type = d.get('coord_type', None)
-        return Kpoints(comment = comment, kpts = kpts, style = style, kpts_shift = kpts_shift, num_kpts = num_kpts)
+        return Kpoints(comment=comment, kpts=kpts, style=style, kpts_shift=kpts_shift, num_kpts=num_kpts)
 
 class PotcarSingle(VaspInput):
     """
@@ -853,7 +875,7 @@ class Potcar(list, VaspInput):
     functional_dir = {'PBE':'POT_GGA_PAW_PBE', 'LDA':'POT_LDA_PAW', 'PW91':'POT_GGA_PAW_PW91'}
     DEFAULT_FUNCTIONAL = "PBE"
 
-    def __init__(self, symbols = None, functional = DEFAULT_FUNCTIONAL, sym_potcar_map = None):
+    def __init__(self, symbols=None, functional=DEFAULT_FUNCTIONAL, sym_potcar_map=None):
         if symbols is not None:
             self.functional = functional
             self.set_symbols(symbols, functional, sym_potcar_map)
@@ -866,7 +888,7 @@ class Potcar(list, VaspInput):
     def from_dict(d):
         functional = d['functional']
         symbols = d['symbols']
-        return Potcar(symbols = symbols, functional = functional)
+        return Potcar(symbols=symbols, functional=functional)
 
     @staticmethod
     def from_file(filename):
@@ -899,7 +921,7 @@ class Potcar(list, VaspInput):
         """
         return [p.symbol for p in self]
 
-    def set_symbols(self, elements, functional = DEFAULT_FUNCTIONAL, sym_potcar_map = None):
+    def set_symbols(self, elements, functional=DEFAULT_FUNCTIONAL, sym_potcar_map=None):
         '''
         Initialize the POTCAR from a set of symbols. Currently, the POTCARs can be fetched from a location specified in pymatgen.cfg or specified explicitly in a map (but not both)
         
@@ -1032,7 +1054,10 @@ class Vasprun(object):
         """
         A complete dos object which incorporates the total dos and all projected dos.
         """
-        return CompleteDos(self.final_structure, self.tdos, self.pdos)
+        final_struct = self.final_structure
+        pdoss = {final_struct[i]:{Orbital.from_vasp_index(j) : self.pdos[i][j]
+                    for j in range(len(self.pdos[i]))} for i in range(len(self.pdos))}
+        return CompleteDos(self.final_structure, self.tdos, pdoss)
 
     @property
     def hubbards(self):
@@ -1049,15 +1074,29 @@ class Vasprun(object):
         else:
             raise VaspParserError("Length of U value parameters and atomic symbols are mismatched")
 
+    @property
+    def run_type(self):
+        """
+        Returns the run type. Currently supports only GGA and HF calcs. 
+        
+        TODO: Fix for other functional types like LDA, PW91, etc.
+        """
+        if self.is_hubbard:
+            return "GGA+U"
+        elif self.parameters.get('LHFCALC', False):
+            return "HF"
+        else:
+            return "GGA"
+
     @property
     def is_hubbard(self):
         if len(self.hubbards) == 0:
             return False
         return sum(self.hubbards.values()) > 0
 
-    def get_band_structure(self, kpoints_filename = None):
+    def get_band_structure(self, kpoints_filename=None, efermi=None):
         """
-        Returns the band structure.
+        Returns the band structure as a BandStructureSymmLine object
         
         Arguments:
             kpoints_filename:
@@ -1065,28 +1104,49 @@ class Vasprun(object):
                 If none is provided, the code will try to intelligently determine the appropriate
                 KPOINTS file by substituting the filename of the vasprun.xml with KPOINTS.
                 The latter is the default behavior.
+            efermi
+                if you want to specify manually the fermi energy this is where you should do it. By default,
+                the None value means the code will get it from the vasprun
                 
         Returns:
-            Band structure object
+            BandStructureSymmLine object
+            
+        TODO:
+        
+        -make a bit more general for non Symm Line band structures
+        -make a decision on the convention with 2*pi or not 
+            
         """
         if not kpoints_filename:
             kpoints_filename = self._filename.replace('vasprun.xml', 'KPOINTS')
         if not os.path.exists(kpoints_filename):
             raise VaspParserError('KPOINTS file needed to obtain band structure.')
+        if not self.incar['ICHARG'] == 11:
+            raise VaspParserError('band structure runs have to be non-self consistent (ICHARG=11)')
+
         k = Kpoints.from_file(kpoints_filename)
         labels_dict = dict(zip(k.labels, k.kpts))
-        lattice_rec = Lattice(self.lattice_rec)
-        #make the labels_dict to work with cartesian
-        if k.coord_type == "Reciprocal":
-            for c in labels_dict:
-                labels_dict[c] = lattice_rec.get_cartesian_coords(2 * math.pi * np.array(labels_dict[c]))
-        kpoints = [lattice_rec.get_cartesian_coords(2 * math.pi * np.array(k)) for k in self.actual_kpoints]
-        eig = self.eigenvalues
+        lattice_new = Lattice(self.lattice_rec.matrix * 2 * math.pi)
+        #lattice_rec=[self.lattice_rec.matrix[i][j] for i,j in range(3)]
+
+        kpoints = [np.array(self.actual_kpoints[i]) for i in range(len(self.actual_kpoints))]
+        dict_eigen = self.to_dict['output']['eigenvalues']
         eigenvals = []
-        for i in range(len(eig[(1, Spin.up)])):
-            eigenvals.append({'energy':[eig[(j + 1, Spin.up)][i][0] for j in range(len(kpoints))],
-                              'occup': [eig[(j + 1, Spin.up)][i][1] for j in range(len(kpoints))]})
-        return Bandstructure(kpoints, eigenvals, labels_dict, lattice_rec, self.final_structure)
+
+        # eigenvalues has a structure of:
+        # {'12':{'up':[[-4.0, 1.0][-3.0, 1.0]]}
+        # we use string version of integer as a keys because of mongoDB
+
+        # Prune off a few eigenvalues to make each kpoint have
+        # the same number of eigenvalues.
+        neigenvalues = [len(v['up']) for k, v in dict_eigen.items()]
+        min_eigenvalues = min(neigenvalues)
+        #max_band = int(math.floor(len(dict_eigen['1']['up']) * 0.9))
+
+        for i in range(min_eigenvalues):
+            eigenvals.append({'energy': [dict_eigen[str(j + 1)]['up'][i][0] for j in range(len(kpoints))]})
+            eigenvals[i]['occup'] = [dict_eigen[str(j + 1)]['up'][i][1] for j in range(len(kpoints))]
+        return BandStructureSymmLine(kpoints, eigenvals, lattice_new, self.efermi, labels_dict)
 
     @property
     def eigenvalue_band_properties(self):
@@ -1172,7 +1232,7 @@ class Vasprun(object):
                 d['output']['eigenvalues'][str(index)][str(spin)] = values
 
         (gap, cbm, vbm, is_direct) = self.eigenvalue_band_properties
-        d['output'].update(dict(bandgap = gap, cbm = cbm, vbm = vbm, is_gap_direct = is_direct))
+        d['output'].update(dict(bandgap=gap, cbm=cbm, vbm=vbm, is_gap_direct=is_direct))
 
         return d
 
@@ -1369,7 +1429,6 @@ class VasprunHandler(xml.sax.handler.ContentHandler):
                         self.kpoints.kpts_shift = [float(x) for x in re.split("\s+", self.val.getvalue().strip())]
                     elif self.state['v'] == "genvec1" or self.state['v'] == "genvec2" or self.state['v'] == "genvec3" or self.state['v'] == "shift":
                         setattr(self.kpoints, self.state['v'], [float(x) for x in re.split("\s+", self.val.getvalue().strip())])
-
         else:
             if self.read_calculation:
                 if name == "i" and self.state['scstep']:
@@ -1605,9 +1664,9 @@ class Outcar(object):
             elif re.search("\((sec|kb)\):", line):
                 tok = line.strip().split(":")
                 run_stats[tok[0].strip()] = float(tok[1].strip())
-            self.run_stats = run_stats
-            self.magnetization = tuple(mag)
-            self.charge = tuple(charge)
+        self.run_stats = run_stats
+        self.magnetization = tuple(mag)
+        self.charge = tuple(charge)
 
     def read_igpar(self):
         """ 
@@ -1777,7 +1836,7 @@ class VolumetricData(object):
     def __sub__(self, other):
         return self.linear_add(other, -1.0)
 
-    def linear_add(self, other, scalefactor = 1.0):
+    def linear_add(self, other, scalefactor=1.0):
         '''
         Method to do a linear sum of volumetric objects.  Use by + and - operators as well.
         '''
@@ -2083,61 +2142,40 @@ class VaspParserError(Exception):
     def __str__(self):
         return "VaspParserError : " + self.msg
 
-def get_band_structure_from_vasp(path):
-    """
-    method taking a directory with a band structure vasp run
-    and returning the corresponding Bandstructure Object
-    also takes into account runs that have been separated in several branches
-    """
 
-    if(os.path.exists(path + "/branch_0")):
-        #get all branches in a list of BandStructurs
-        list_branches = []
-        listdir_clean = []
-        for f in os.listdir(path):
-            if "branch" in f:
-                listdir_clean.append(f)
-        for i in range(len(listdir_clean)):
-            for f in listdir_clean:
-                if(int(f.split("_")[1]) == i):
-                    list_branches.append(get_band_structure_from_vasp_individual(path + "/" + f))
-        return get_reconstructed_band_structure(list_branches)
+def get_band_structure_from_vasp_multiple_branches(dir_name, efermi=None):
+    """
+    this method is used to get band structure info from a VASP directory. It takes into account that
+    the run can be divided in several branches named "branch_x". If the run has not been divided in branches
+    the method will turn to parsing vasprun.xml directly.
+
+    The method returns None is there's a parsing error
+    """
+    #ToDo: Add better error handling!!!
+    if os.path.exists(os.path.join(dir_name, "branch_0")):
+        #get all branch dir names
+        branch_dir_names = [os.path.abspath(d) for d in glob.glob("{i}/branch_*".format(i=dir_name)) if os.path.isdir(d)]
+
+        #sort by the directory name (e.g, branch_10)
+        sort_by = lambda x: int(x.split('_')[-1])
+        sorted_branch_dir_names = sorted(branch_dir_names, key=sort_by)
+
+        # populate branches with Bandstructure instances
+        branches = []
+        for dir_name in sorted_branch_dir_names:
+            xml_file = os.path.join(dir_name, 'vasprun.xml')
+            if os.path.exists(xml_file):
+                run = Vasprun(xml_file)
+                # why are these keyword args?
+                branches.append(run.get_band_structure(kpoints_filename=None, efermi=efermi))
+            else:
+                # It might be better to throw an exception
+                warnings.warn("Skipping {d}. Unable to find {f}".format(d=dir_name, f=xml_file))
+        return get_reconstructed_band_structure(branches, efermi)
     else:
-        return get_band_structure_from_vasp_individual(path)
-
-def get_band_structure_from_vasp_individual(path):
-    run = Vasprun(path + "/vasprun.xml")
-    labels_dict = parse_kpoint_labels(path + "/KPOINTS")
-    lattice_rec = run.final_structure.lattice.reciprocal_lattice
-    for c in labels_dict:
-        labels_dict[c] = lattice_rec.get_cartesian_coords(labels_dict[c])
-    kpoints = [lattice_rec.get_cartesian_coords(np.array(run.actual_kpoints[i])) for i in range(len(run.actual_kpoints))]
-    dict_eigen = run.to_dict['output']['eigenvalues']
-    eigenvals = []
-    max_band = int(math.floor(len(dict_eigen['1']['up']) * 0.9))
-    for i in range(max_band):
-        eigenvals.append({'energy':[dict_eigen[str(j + 1)]['up'][i][0] for j in range(len(kpoints))]})
-        eigenvals[i]['occup'] = [dict_eigen[str(j + 1)]['up'][i][1] for j in range(len(kpoints))]
-    bands = Bandstructure(kpoints, eigenvals, labels_dict, run.final_structure, run.efermi)
-    return bands
-
-def parse_kpoint_labels(file_kpoints):
-    """
-    helper method to parse a kpoint file and get the kpoint labels
-    """
-    with file_open_zip_aware(file_kpoints, "r") as f:
-        lines = f.readlines()
-    count = -1
-    dict_label_kpoints = {}
-    for line in lines:
-        count = count + 1
-        if(count < 4):
-            continue
-
-        tokens = re.split(" +", line)
-        if len(tokens) < 6:
-            continue
-        array = np.array([float(tokens[1]), float(tokens[2]), float(tokens[3])])
-        dict_label_kpoints[tokens[5]] = array
-
-    return dict_label_kpoints
+        xml_file = os.path.join(dir_name, 'vasprun.xml')
+        #Better handling of Errors
+        if os.path.exists(xml_file):
+            return Vasprun(xml_file).get_band_structure(kpoints_filename=None, efermi=efermi)
+        else:
+            return None
